package org.gcube.portlets.user.geoportaldataviewer.client;

import java.util.List;

import org.gcube.application.geoportalcommon.shared.GeoNaDataViewerProfile;
import org.gcube.application.geoportalcommon.shared.GeoNaItemRef;
import org.gcube.application.geoportalcommon.shared.gis.BoundsMap;
import org.gcube.application.geoportalcommon.shared.products.ConcessioneDV;
import org.gcube.application.geoportalcommon.shared.products.model.LayerConcessioneDV;
import org.gcube.application.geoportalcommon.shared.products.model.UploadedImageDV;
import org.gcube.portlets.user.geoportaldataviewer.shared.GeoNaSpatialQueryResult;
import org.gcube.portlets.user.geoportaldataviewer.shared.gis.BaseMapLayer;
import org.gcube.portlets.user.geoportaldataviewer.shared.gis.LayerObject;
import org.gcube.portlets.user.geoportaldataviewer.shared.gis.wms.GeoInformationForWMSRequest;

import com.google.gwt.user.client.rpc.RemoteService;
import com.google.gwt.user.client.rpc.RemoteServiceRelativePath;

/**
 * The client side stub for the RPC service.
 *
 * @author Francesco Mangiacrapa at ISTI-CNR (francesco.mangiacrapa@isti.cnr.it)
 * 
 *         Nov 17, 2020
 */
@RemoteServiceRelativePath("geoportaldataviewerservice")
public interface GeoportalDataViewerService extends RemoteService {

	/**
	 * Parses the wms request.
	 *
	 * @param wmsRequest the wms request
	 * @param layerName  the layer name
	 * @return the geo information for WMS request
	 * @throws Exception the exception
	 */
	GeoInformationForWMSRequest parseWmsRequest(String wmsRequest, String layerName) throws Exception;

	/**
	 * Gets the data result.
	 *
	 * @param layerObjects  the layer objects
	 * @param mapSrsName    the map srs name
	 * @param mapBBOX       the map BBOX
	 * @param maxWFSFeature the max WFS feature
	 * @param zoomLevel     the zoom level
	 * @return the data result
	 */
	List<GeoNaSpatialQueryResult> getDataResult(List<LayerObject> layerObjects, String mapSrsName, BoundsMap mapBBOX,
			int maxWFSFeature, double zoomLevel);

	/**
	 * Gets the concessione for id.
	 *
	 * @param mongoId the mongo id
	 * @return the concessione for id
	 * @throws Exception the exception
	 */
	ConcessioneDV getConcessioneForId(String mongoId) throws Exception;

	/**
	 * Gets the my login.
	 *
	 * @return the my login
	 */
	String getMyLogin();

	/**
	 * Gets the layer for type.
	 *
	 * @param layerType the layer type
	 * @return the layer for type
	 * @throws Exception the exception
	 */
	GeoInformationForWMSRequest getLayerForType(String layerType) throws Exception;

	/**
	 * Gets the geo na data view profile.
	 *
	 * @return the geo na data view profile
	 * @throws Exception the exception
	 */
	GeoNaDataViewerProfile getGeoNaDataViewProfile() throws Exception;

	/**
	 * Gets the uploaded images for id.
	 *
	 * @param itemType the item type
	 * @param itemId the item id
	 * @param maxImages the max images
	 * @return the uploaded images for id
	 * @throws Exception 
	 */
	List<UploadedImageDV> getUploadedImagesForId(String itemType, String itemId, Integer maxImages) throws Exception;

	/**
	 * Gets the public links for.
	 *
	 * @param item the item
	 * @return the public links for
	 * @throws Exception the exception
	 */
	GeoNaItemRef getPublicLinksFor(GeoNaItemRef item) throws Exception;

	/**
	 * Gets the layers for id.
	 *
	 * @param itemType  the item type
	 * @param itemId    the item id
	 * @return the layers for id
	 * @throws Exception the exception
	 */
	List<LayerConcessioneDV> getLayersForId(String itemType, String itemId) throws Exception;

	/**
	 * Gets the WFS features.
	 *
	 * @param layerObjects  the layer objects
	 * @param mapSrsName    the map srs name
	 * @param selectBBOX    the select BBOX
	 * @param maxWFSFeature the max WFS feature
	 * @param zoomLevel     the zoom level
	 * @return the WFS features
	 */
	List<GeoNaSpatialQueryResult> getWFSFeatures(List<LayerObject> layerObjects, String mapSrsName,
			BoundsMap selectBBOX, int maxWFSFeature, double zoomLevel);

	List<BaseMapLayer> getListBaseLayers();


}
