/**
 *
 */
package org.gcube.portlets.user.geoportaldataviewer.server.util;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.gcube.common.authorization.library.provider.SecurityTokenProvider;
import org.gcube.common.portal.PortalContext;
import org.gcube.common.scope.api.ScopeProvider;
import org.gcube.portlets.user.geoportaldataviewer.server.GNABaseMapsResourceReader;
import org.gcube.portlets.user.geoportaldataviewer.shared.gis.BaseMapLayer;
import org.gcube.portlets.user.urlshortener.UrlShortener;
import org.gcube.vomanagement.usermanagement.GroupManager;
import org.gcube.vomanagement.usermanagement.exception.GroupRetrievalFault;
import org.gcube.vomanagement.usermanagement.exception.UserManagementSystemException;
import org.gcube.vomanagement.usermanagement.impl.LiferayGroupManager;
import org.gcube.vomanagement.usermanagement.model.GCubeGroup;
import org.gcube.vomanagement.usermanagement.model.GCubeUser;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.liferay.portal.service.UserLocalServiceUtil;

/**
 * The Class SessionUtil.
 *
 * @author Francesco Mangiacrapa at ISTI-CNR (francesco.mangiacrapa@isti.cnr.it)
 * 
 *         Oct 20, 2020
 */
public class SessionUtil {

	/** The log. */
	private static Logger LOG = LoggerFactory.getLogger(SessionUtil.class);

	private static final String URL_SHORTENER_SERVICE = "URL_SHORTENER_SERVICE";
	private static final String LIST_BASE_MAPS_LAYERS = "LIST_BASE_MAPS_LAYERS";

	/**
	 * Checks if is into portal.
	 *
	 * @return true, if is into portal
	 */
	public static boolean isIntoPortal() {
		try {
			UserLocalServiceUtil.getService();
			return true;
		} catch (Exception ex) {
			LOG.warn("Development Mode ON");
			return false;
		}
	}

	/**
	 * Checks if is session expired.
	 *
	 * @param httpServletRequest the http servlet request
	 * @return true, if is session expired
	 * @throws Exception the exception
	 */
	public static boolean isSessionExpired(HttpServletRequest httpServletRequest) throws Exception {
		LOG.trace("workspace session validating...");
		return PortalContext.getConfiguration().getCurrentUser(httpServletRequest) == null;
	}

	/**
	 * Retrieve the current user by using the portal manager.
	 *
	 * @param request the request
	 * @return a GcubeUser object
	 */
	public static GCubeUser getCurrentUser(HttpServletRequest request) {

		if (request == null)
			throw new IllegalArgumentException("HttpServletRequest is null!");

		PortalContext pContext = PortalContext.getConfiguration();
		GCubeUser user = pContext.getCurrentUser(request);
		LOG.debug("Returning user " + user);
		return user;
	}

	/**
	 * Retrieve the current scope by using the portal manager.
	 *
	 * @param request     the request
	 * @param setInThread the set in thread
	 * @return a GcubeUser object
	 */
	public static String getCurrentContext(HttpServletRequest request, boolean setInThread) {

		if (request == null)
			throw new IllegalArgumentException("HttpServletRequest is null!");

		PortalContext pContext = PortalContext.getConfiguration();
		String context = pContext.getCurrentScope(request);

		if (context != null && setInThread)
			ScopeProvider.instance.set(context);

		LOG.debug("Returning context " + context);
		return context;
	}

	/**
	 * Retrieve the current scope by using the portal manager.
	 *
	 * @param request     the request
	 * @param setInThread the set in thread
	 * @return a GcubeUser object
	 */
	public static String getCurrentToken(HttpServletRequest request, boolean setInThread) {

		if (request == null)
			throw new IllegalArgumentException("HttpServletRequest is null!");

		PortalContext pContext = PortalContext.getConfiguration();
		String scope = pContext.getCurrentScope(request);
		GCubeUser user = pContext.getCurrentUser(request);
		String token = PortalContext.getConfiguration().getCurrentUserToken(scope, user.getUsername());

		if (token != null && setInThread)
			SecurityTokenProvider.instance.set(token);

		LOG.debug("Returning token " + token.substring(1, 10) + "_MASKED_TOKEN_");
		return token;
	}

	/**
	 * Retrieve the group given the scope.
	 *
	 * @param scope the scope
	 * @return the group from scope
	 * @throws UserManagementSystemException the user management system exception
	 * @throws GroupRetrievalFault           the group retrieval fault
	 */
	public static GCubeGroup getGroupFromScope(String scope) throws UserManagementSystemException, GroupRetrievalFault {

		if (scope == null || scope.isEmpty())
			throw new IllegalArgumentException("Scope is missing here!!");

		GroupManager gm = new LiferayGroupManager();
		long groupId = gm.getGroupIdFromInfrastructureScope(scope);
		return gm.getGroup(groupId);

	}

	/**
	 * Gets the url shortener.
	 *
	 * @param httpServletRequest the http servlet request
	 * @return the url shortener
	 */
	public static UrlShortener getUrlShortener(HttpServletRequest httpServletRequest) {

		HttpSession session = httpServletRequest.getSession();
		UrlShortener shortener = null;
		try {
			shortener = (UrlShortener) session.getAttribute(URL_SHORTENER_SERVICE);

			if (shortener == null) {
				shortener = new UrlShortener();
				session.setAttribute(URL_SHORTENER_SERVICE, shortener);
			}

		} catch (Exception e) {
			LOG.warn("Error occurred when instancing the " + UrlShortener.class.getSimpleName(), e);
		}

		return shortener;
	}

	/**
	 * Gets the GNA base maps.
	 *
	 * @param httpServletRequest the http servlet request
	 * @return the GNA base maps
	 */
	public static List<BaseMapLayer> getGNABaseMaps(HttpServletRequest httpServletRequest) {
		HttpSession session = httpServletRequest.getSession();
		List<BaseMapLayer> lstBML = null;
		try {
			lstBML = (List<BaseMapLayer>) session.getAttribute(LIST_BASE_MAPS_LAYERS);

			if (lstBML == null) {
				GNABaseMapsResourceReader gnaBMRR = new GNABaseMapsResourceReader();
				lstBML = gnaBMRR.getListBaseMaps();
				session.setAttribute(LIST_BASE_MAPS_LAYERS, lstBML);
			}

		} catch (Exception e) {
			LOG.warn("Error occurred reading the base maps by " + GNABaseMapsResourceReader.class.getSimpleName(), e);
		}
		return lstBML;
	}
}
